/*
 * Copyright (c) 2001 Fenris, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * File:  display_list.h
 *
 * Purpose:  This header file declares a special list for display objects 
 * and the interface for its use.
 *
 */
#ifndef DSPLIST_H
#define DSPLIST_H


#include <pthread.h>


typedef struct _Dsp_Node {
    void * 		obj;
    struct _Dsp_Node * 	next;
} Dsp_Node;

typedef struct {
    Dsp_Node * 		head;
    Dsp_Node * 		tail;
    unsigned int 	count;
} Dsp_List;


void init_dsp_list (Dsp_List* list);
void append_dsp_list (Dsp_List* list, void* obj);
void insert_src_dsp_obj (Dsp_List* list, void* obj);
void insert_dst_dsp_obj (Dsp_List* list, void* obj);
void* delete_dsp_list_head (Dsp_List* list);
void* delete_dsp_list_node (Dsp_List* list, Dsp_Node* node);
void empty_dsp_list (Dsp_List* list);


#endif /* DSPLIST_H */
