/*
 * Copyright (c) 2001 Fenris, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * File:  filter.h
 *
 * Purpose:  This header file declares the enumerated types necessary to 
 * match the gui interface input to a global filter object that can
 * be accessed by the filter (see filter.c).
 *
 */
#ifndef FILTER_H
#define FILTER_H
 

#include <stdlib.h>

#include "bool.h"
#include "headers.h"
#include "network.h"

#define NUM_OF_PORTS 65536


enum Menu {all, arp_m, ip_m};
enum Arp_Menu {arp_all};
enum Ip_Menu {ip_all, ip_port, icmp_m, igmp_m, tcp_m, udp_m};
enum Icmp_Menu {icmp_all, icmp_rqst, icmp_rply, icmp_pair};
enum Igmp_Menu {igmp_all};
enum Tcp_Menu {tcp_all, tcp_port, tcp_http, tcp_ftp, tcp_ssh};
enum Udp_Menu {udp_all, udp_port};


union IP_Submenu {
    enum Icmp_Menu 		icmp;
    enum Igmp_Menu 		igmp;
    enum Tcp_Menu 		tcp;
    enum Udp_Menu 		udp;
};

typedef struct {
    enum Arp_Menu 		arp;
    enum Ip_Menu 		ip;
    union IP_Submenu  		ip_submenu;
    int				src_addr;
    int				dst_addr;
    enum bool			filter_on_src_addr;
    enum bool			filter_on_dst_addr;
    int				src_cidr;
    int				dst_cidr;
    enum bool			filter_on_src_ports;
    enum bool			filter_on_dst_ports;
    enum bool 			src_ports[NUM_OF_PORTS + 1];
    enum bool 			dst_ports[NUM_OF_PORTS + 1];
} Net_Opts;

typedef struct {    
    enum Menu 			menu;
    Net_Opts 			net_submenu;
    int                         db;
    int                         gui;
} Filter_Opts;


void filter(Packet_Headers* p, Net_Obj** obj);
enum bool valid_arp_addr(Packet_Headers* p, Filter_Opts* filter_opts);
enum bool valid_ip_addr(Packet_Headers* p, Filter_Opts* filter_opts);
enum bool valid_port(Packet_Headers* p, Filter_Opts* filter_opts);
void create_arp_obj(Packet_Headers* p, Net_Obj** obj);
void create_ip_obj(Packet_Headers* p, Net_Obj** obj);


#endif /* FILTER_H */
