/*
 * Copyright (c) 2001 Fenris, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * File:  network.h
 *
 * Purpose:  This header file declares the network object, and the interface
 * to its use.  For further information see network.c
 *
 */
#ifndef NETWORK_DATA_H
#define NETWORK_DATA_H


#include <pthread.h>
#include <sys/time.h>

#include "network_list.h"

#define SRC 0
#define DST 1
#define PAIR 2


typedef struct {
    unsigned int src;
    unsigned int dst;
    struct timeval ts;
    unsigned int len;
} Net_Obj;

typedef struct {
    Net_List* src_list;
    Net_List* dst_list;
    Net_List* pair_list;
    int timeout;
} Network;


int init_net(Network* net);
void into_net(Network* net, Net_Obj* obj);
void refresh_net(Network* net);
void check_resolver(Network *net);
int net_sublist_clear(Net_List* list, Net_Node* current, struct timeval tc);
int net_check_sublists(Net_List* netlist, Net_Obj* obj, int flag);
void empty_net(Network* net);


#endif /* NETWORK_DATA_H */
