/*
 * Copyright (c) 2001 Fenris, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * File:  network_list.h
 *
 * Purpose:  This header file contains the declarations and interface for
 * a custom list used to handle network objects as declared in network.h
 *
 */
#ifndef NETLIST_H
#define NETLIST_H


#include <pthread.h>

#define MAX_HOSTNAME_LEN 64


typedef struct _Net_Node {
    void * 			obj;
    struct _Net_Node * 		next;
} Net_Node;


typedef struct {
    Net_Node * 			head;
    Net_Node * 			tail;
    unsigned int 		count;
    unsigned int 		size;
    char   			src[MAX_HOSTNAME_LEN];
    char  			dst[MAX_HOSTNAME_LEN];
} Net_List;


void init_net_list (Net_List* list);
void append_net_list (Net_List* list, void* obj);
void* delete_net_list_head (Net_List* list);
void* delete_net_list_node (Net_List* list, Net_Node* node);
void empty_net_list (Net_List* list);


#endif /* NETLIST_H */
