<?php

if ($map) {
    $pie = new pie($a, $g);
    $pie->imagemap($HTTP_GET_VARS);
} else {
    $pie = new pie($HTTP_GET_VARS, $g);
    $pie->imagepng();
}


class pie
{
    var $size = 500;
    var $diam = 350;
    var $fn = 2;
    var $a;
    var $cx;
    var $cy;
    var $total;

    function pie($a = array(), $gran = 2)
    {
        $this->cx = $this->size / 2;
        $this->cy = $this->size / 2;
        $this->total = array_sum($a);
        $in = $a;

        if ($this->total <= 0)
            return;

        # Determine our element granularity.
        if ($gran < 1) $gran = 1;
        if ($gran > 10) $gran = 10;

        # Remove elements which are too small to display properly.
        asort($a);
        do {
            reset($a);
            list($k, $v) = each($a);
            if (($v / $this->total * 360) < $gran) {
                $this->total -= array_shift($a);
                $pop = true;
            } else {
                $pop = false;
            }
        } while ($pop);

        # Put remaining array elements back into passed-in order.
        reset($a);
        while (list($k, $v) = each($in)) {
            if (isset($a[$k]))
                $this->a[$k] = $a[$k];
        }

        # Convert values to degrees.
        $totaldeg = $this->total / 360;
        reset ($this->a);
        while (list($k, $v) = each($this->a))
            $this->a[$k] /= $totaldeg;
    }

    function imagemap($args = array())
    {
        $radius = $this->diam / 2;
        $cx = $this->cx;
        $cy = $this->cy;

        if (array_sum($this->a) <= 0)
            return;

        $s = '';
        for (reset($args); list($k, $v) = each($args);) {
            if ($k == 't' || $k == 'show') continue;
            $s .= urlencode($k) . '=' . urlencode($v) . '&';
        }

        $cur = 0;
        $i = 0;
        reset($this->a);
        while (list($k, $v) = each($this->a)) {
            echo "<area shape=\"poly\" alt=\"$k\" href=\"./?{$s}t=$k\" coords=\"";
            echo $this->cx . ',' . $this->cy . ',';

            for ($i = $cur; $i < ($cur + $v); $i += 10)
                echo $this->area($i) . ',';
            
            echo $this->area($cur + $v) . "\" />\n";

            $cur += $v;
            ++$i;
        }
    }

    function area($ang)
    {
        $radius = $this->diam / 2;
        $cx = $this->cx;
        $cy = $this->cy;
        $theta = deg2rad($ang);

        return round($cx + $radius * cos($theta)) . ','
            . round($cy + $radius * sin($theta));
    }

    function imagepng()
    {
        header('Content-type: image/png');

        if (array_sum($this->a) <= 0) {
            $msg = "No packets in database.";
            $im = imagecreate(imagefontwidth($this->fn) * strlen($msg) + 8,
                              imagefontheight($this->fn) + 4);
            $bg = imagecolorallocate($im, 255, 255, 255);
            imagecolortransparent($im, $bg);
            $c = imagecolorallocate($im, 0, 0, 0);
            imagestring($im, $this->fn, 4, 2, $msg, $c);
            imagepng($im);
            imagedestroy($im);
            return;
        }

        $im = imagecreate($this->size, $this->size);
        $bg = imagecolorallocate($im, 255, 255, 255);
        imagecolortransparent($im, $bg);
        $black = imagecolorallocate($im, 0, 0, 0);

        $c[] = imagecolorallocate($im, 255, 0, 0);          # red
        $c[] = imagecolorallocate($im, 0, 255, 0);          # green
        $c[] = imagecolorallocate($im, 0, 0, 255);          # blue
        $c[] = imagecolorallocate($im, 0, 255, 255);        # cyan
        $c[] = imagecolorallocate($im, 255, 0, 255);        # magenta
        $c[] = imagecolorallocate($im, 255, 255, 0);        # yellow

        # Draw a full circle first; for some reason we get spillage otherwise.
        imagearc($im, $this->cx, $this->cy,
                 $this->diam, $this->diam, 0, 360, $black);
        imagestring($im, 2, 0, 0, "$this->total packets displayed.", $black);

        $cur = 0;
        $i = 0;
        reset($this->a);
        while (list($k, $v) = each($this->a)) {
            if ($i >= count($c))
                $i = 0;
            $this->draw_wedge($im, $k, $cur, $cur + $v, $c[$i], $black);
            $cur += $v;
            if (++$i >= count($c))
                $i = 0;
        }

        imagepng($im);
        imagedestroy($im);
        return;
    }

    function draw_wedge($im, $str, $a, $b, $fc, $bc)
    {
        $theta_a  = deg2rad($a);
        $theta_b  = deg2rad($b);
        $theta_ab = deg2rad(($a + $b) / 2);
        $radius   = $this->diam / 2;
        $cx = $this->cx;
        $cy = $this->cy;

        $x = $cx + $radius * cos($theta_a);
        $y = $cy + $radius * sin($theta_a);
        imageline($im, $cx, $cy, $x, round($y), $bc);

        $x = $cx + $radius * cos($theta_b);
        $y = $cy + $radius * sin($theta_b);
        imageline($im, $cx, $cy, $x, round($y), $bc);

        $x = $cx + $radius * .9 * cos($theta_ab);
        $y = $cy + $radius * .9 * sin($theta_ab);
        imagefill($im, $x, $y, $fc);

        $this->label_wedge($im, $str, $theta_ab, $fc);
    }

    function label_wedge($im, $str, $theta, $fc)
    {
        $radius = $this->diam / 2;
        $cx = $this->cx;
        $cy = $this->cy;

        $width  = imagefontwidth($this->fn) * strlen($str);
        $height = imagefontheight($this->fn) / 2;
        $radius += $height * 2;

        $x = $radius * cos($theta) - (1 - cos($theta)) * $width / 2;
        $y = ($radius + $height) * sin($theta);

        $x += $cx;
        $y += $cy;

        $str = ereg_replace('_', '.', $str);
        imagestring($im, $this->fn, $x, $y - $height, $str, $fc);
    }
}

?>
